@@ -69,6 +69,7 @@ from _drgn import (
     Symbol,
     SymbolBinding,
     SymbolKind,
+    Thread,
     Type,
     TypeEnumerator,
     TypeKind,
@@ -114,6 +115,7 @@ __all__ = (
     "Symbol",
     "SymbolBinding",
     "SymbolKind",
+    "Thread",
     "Type",
     "TypeEnumerator",
     "TypeKind",
