@@ -85,9 +85,7 @@ def list_first_entry_or_null(
     head = head.read_()
     pos = head.next.read_()
     if pos == head:
-        if isinstance(type, str):
-            type = head.prog_.type(type)
-        return NULL(head.prog_, head.prog_.pointer_type(type))
+        return NULL(head.prog_, head.prog_.pointer_type(head.prog_.type(type)))
     else:
         return container_of(pos, type, member)
 
@@ -167,6 +165,7 @@ def list_for_each_entry(
     :param member: Name of list node member in entry type.
     :return: Iterator of ``type *`` objects.
     """
+    type = head.prog_.type(type)
     for pos in list_for_each(head):
         yield container_of(pos, type, member)
 
@@ -182,6 +181,7 @@ def list_for_each_entry_reverse(
     :param member: Name of list node member in entry type.
     :return: Iterator of ``type *`` objects.
     """
+    type = head.prog_.type(type)
     for pos in list_for_each_reverse(head):
         yield container_of(pos, type, member)
 
@@ -219,5 +219,6 @@ def hlist_for_each_entry(
     :param member: Name of list node member in entry type.
     :return: Iterator of ``type *`` objects.
     """
+    type = head.prog_.type(type)
     for pos in hlist_for_each(head):
         yield container_of(pos, type, member)
