@@ -168,6 +168,7 @@ def rbtree_inorder_for_each_entry(
     :param member: Name of red-black node member in entry type.
     :return: Iterator of ``type *`` objects.
     """
+    type = root.prog_.type(type)
     for node in rbtree_inorder_for_each(root):
         yield container_of(node, type, member)
 
@@ -197,6 +198,8 @@ def rb_find(
         the key matches the entry.
     :return: ``type *`` found entry, or ``NULL`` if not found.
     """
+    prog = root.prog_
+    type = prog.type(type)
     node = root.rb_node.read_()
     while node:
         entry = container_of(node, type, member)
@@ -207,4 +210,4 @@ def rb_find(
             node = node.rb_right.read_()
         else:
             return entry
-    return NULL(root.prog_, type)
+    return NULL(prog, prog.pointer_type(type))
