@@ -36,7 +36,7 @@
 /** Minor version of drgn. */
 #define DRGN_VERSION_MINOR 0
 /** Patch level of drgn. */
-#define DRGN_VERSION_PATCH 16
+#define DRGN_VERSION_PATCH 17
 
 /**
  * @defgroup ErrorHandling Error handling
@@ -252,6 +252,25 @@ void drgn_error_destroy(struct drgn_error *err);
 
 /** @} */
 
+
+/**
+ * @defgroup Languages Languages
+ *
+ * Programming languages.
+ *
+ * @{
+ */
+
+/** C */
+extern const struct drgn_language drgn_language_c;
+/** C++ */
+extern const struct drgn_language drgn_language_cpp;
+
+/** Get the name of a @ref drgn_language. */
+const char *drgn_language_name(const struct drgn_language *lang);
+
+/** @} */
+
 /**
  * @ingroup Types
  *
@@ -420,6 +439,7 @@ const char * const *drgn_register_names(const struct drgn_register *reg,
 /** @} */
 
 struct drgn_object;
+struct drgn_thread;
 
 /**
  * @defgroup Programs Programs
@@ -693,6 +713,10 @@ const struct drgn_platform *drgn_program_platform(struct drgn_program *prog);
 /** Get the default language of a @ref drgn_program. */
 const struct drgn_language *drgn_program_language(struct drgn_program *prog);
 
+/** Set the default language of a @ref drgn_program. */
+void drgn_program_set_language(struct drgn_program *prog,
+			       const struct drgn_language *lang);
+
 /**
  * Read from a program's memory.
  *
@@ -817,6 +841,36 @@ struct drgn_error *drgn_program_find_symbol_by_name(struct drgn_program *prog,
 						    const char *name,
 						    struct drgn_symbol **ret);
 
+/**
+ * Get all global and local symbols, optionally with the given name.
+ *
+ * @param[in] prog Program.
+ * @param[in] name Name to match. If @c NULL, returns all symbols.
+ * @param[out] syms_ret Returned array of symbols. On success, this must be
+ * freed with @ref drgn_symbols_destroy().
+ * @param[out] count_ret Returned number of symbols in @p syms_ret.
+ * @return @c NULL on success, non-@c NULL on error.
+ */
+struct drgn_error *drgn_program_find_symbols_by_name(struct drgn_program *prog,
+						     const char *name,
+						     struct drgn_symbol ***syms_ret,
+						     size_t *count_ret);
+
+/**
+ * Get all symbols containing the given address.
+ *
+ * @param[in] prog Program.
+ * @param[in] address Address to search for.
+ * @param[out] syms_ret Returned array of symbols. On success, this must be
+ * freed with @ref drgn_symbols_destroy().
+ * @param[out] count_ret Returned number of symbols in @p syms_ret.
+ * @return @c NULL on success, non-@c NULL on error.
+ */
+struct drgn_error *drgn_program_find_symbols_by_address(struct drgn_program *prog,
+							uint64_t address,
+							struct drgn_symbol ***syms_ret,
+							size_t *count_ret);
+
 /** Element type and size. */
 struct drgn_element_info {
 	/** Type of the element. */
@@ -2599,6 +2653,16 @@ enum drgn_symbol_kind {
 /** Destroy a @ref drgn_symbol. */
 void drgn_symbol_destroy(struct drgn_symbol *sym);
 
+/**
+ * Destroy each @ref drgn_symbol in @syms, and free the array.
+ *
+ * This will ignore any @c NULL entry in the array, allowing you to take
+ * ownership of any symbol from the array prior to freeing the rest. For each
+ * symbol you take ownership of, you must free it with @ref
+ * drgn_symbol_destroy().
+ */
+void drgn_symbols_destroy(struct drgn_symbol **syms, size_t count);
+
 /**
  * Get the name of a @ref drgn_symbol.
  *
@@ -2754,4 +2818,122 @@ struct drgn_error *drgn_object_stack_trace(const struct drgn_object *obj,
 
 /** @} */
 
+/**
+ * @defgroup Threads Threads
+ *
+ * Threads in a program.
+ *
+ * @{
+ */
+
+/**
+ * @struct drgn_thread
+ *
+ * A thread in a program.
+ */
+struct drgn_thread;
+
+/**
+ * Create a copy of a @ref drgn_thread.
+ *
+ * @param[in] thread Thread to copy.
+ * @param[out] ret Returned copy. On success, must be destroyed with @ref
+ * drgn_thread_destroy().
+ * @return @c NULL on success, non-@c NULL on error.
+ */
+struct drgn_error *drgn_thread_dup(const struct drgn_thread *thread,
+				   struct drgn_thread **ret);
+
+/** Free a @ref drgn_thread. */
+void drgn_thread_destroy(struct drgn_thread *thread);
+
+/**
+ * @struct drgn_thread_iterator
+ *
+ * An iterator over all the threads in a program.
+ */
+struct drgn_thread_iterator;
+
+/**
+ * Get an iterator over all of the threads in the program.
+ *
+ * @param[out] ret Returned iterator, which can be advanced with @ref
+ * drgn_thread_iterator_next, and must be destroyed with @ref
+ * drgn_thread_iterator_destroy.
+ * @return @c NULL on success, non-@c NULL on error.
+ */
+struct drgn_error *
+drgn_thread_iterator_create(struct drgn_program *prog,
+			    struct drgn_thread_iterator **ret);
+
+/** Free a @ref drgn_thread_iterator. */
+void drgn_thread_iterator_destroy(struct drgn_thread_iterator *it);
+
+/**
+ * Get the next thread from a @ref drgn_thread_iterator.
+ *
+ * @param[out] ret Borrowed thread handle, or @c NULL if there are no more
+ * threads. This is valid until until the next call to @ref
+ * drgn_thread_iterator_next() with the same @p it, or until @p it is destroyed.
+ * It may be copied with @ref drgn_thread_dup() if it is needed for longer. This
+ * must NOT be destroyed with @ref drgn_thread_destroy().
+ * @return @c NULL on success, non-@c NULL on error.
+ */
+struct drgn_error *drgn_thread_iterator_next(struct drgn_thread_iterator *it,
+					     struct drgn_thread **ret);
+
+/**
+ * Get the thread with the given thread ID.
+ *
+ * @param[in] tid Thread ID.
+ * @param[out] ret New thread handle, or @c NULL if not found. On success, must
+ * be destroyed with @ref drgn_thread_destroy().
+ * @return @c NULL on success, non-@c NULL on error.
+ */
+struct drgn_error *drgn_program_find_thread(struct drgn_program *prog,
+					    uint32_t tid,
+					    struct drgn_thread **ret);
+
+/**
+ * Get the main program thread.
+ *
+ * @param[out] ret Borrowed thread handle. This is valid for the lifetime of @p
+ * prog. This must NOT be destroyed with @ref drgn_thread_destroy().
+ * @return @c NULL on success, non-@c NULL on error.
+ */
+struct drgn_error *drgn_program_main_thread(struct drgn_program *prog,
+					    struct drgn_thread **ret);
+
+/**
+ * Get the thread that caused the program to crash.
+ *
+ * @param[out] ret Borrowed thread handle. This is valid for the lifetime of @p
+ * prog. This must NOT be destroyed with @ref drgn_thread_destroy().
+ * @return @c NULL on success, non-@c NULL on error.
+ */
+struct drgn_error *drgn_program_crashed_thread(struct drgn_program *prog,
+					       struct drgn_thread **ret);
+
+/**
+ * Get the object for the given thread. This is currently only defined for the
+ * Linux kernel.
+ *
+ * @param[out] ret Returned object. This must not be modified and is valid for
+ * the lifetime of @p thread. It can be copied with @ref drgn_object_copy() if
+ * it is needed for longer.
+ * @return @c NULL on success, non-@c NULL on error.
+ */
+struct drgn_error *drgn_thread_object(struct drgn_thread *thread,
+				      const struct drgn_object **ret);
+
+/**
+ * Get a stack trace for the thread represented by @p thread.
+ *
+ * @sa drgn_program_stack_trace().
+ */
+struct drgn_error *drgn_thread_stack_trace(struct drgn_thread *thread,
+					   struct drgn_stack_trace **ret);
+
+/** @} */
+
 #endif /* DRGN_H */
