@@ -6,9 +6,11 @@
 #include <inttypes.h>
 
 #include "drgn.h"
+#include "helpers.h"
 #include "minmax.h"
 #include "platform.h"
 #include "program.h"
+#include "util.h"
 
 struct drgn_error *linux_helper_read_vm(struct drgn_program *prog,
 					uint64_t pgtable, uint64_t virt_addr,
@@ -99,6 +101,67 @@ struct drgn_error *linux_helper_read_vm(struct drgn_program *prog,
 	return err;
 }
 
+struct drgn_error *linux_helper_per_cpu_ptr(struct drgn_object *res,
+					    const struct drgn_object *ptr,
+					    uint64_t cpu)
+{
+	struct drgn_error *err;
+	struct drgn_program *prog = drgn_object_program(ptr);
+
+	struct drgn_object tmp;
+	drgn_object_init(&tmp, prog);
+	err = drgn_program_find_object(prog, "__per_cpu_offset", NULL,
+				       DRGN_FIND_OBJECT_ANY, &tmp);
+	if (!err) {
+		err = drgn_object_subscript(&tmp, &tmp, cpu);
+		if (err)
+			goto out;
+		union drgn_value per_cpu_offset;
+		err = drgn_object_read_integer(&tmp, &per_cpu_offset);
+		if (err)
+			goto out;
+
+		uint64_t ptr_value;
+		err = drgn_object_read_unsigned(ptr, &ptr_value);
+		if (err)
+			goto out;
+
+		err = drgn_object_set_unsigned(res,
+					       drgn_object_qualified_type(ptr),
+					       ptr_value + per_cpu_offset.uvalue,
+					       0);
+	} else if (err->code == DRGN_ERROR_LOOKUP) {
+		drgn_error_destroy(err);
+		err = drgn_object_copy(res, ptr);
+	}
+out:
+	drgn_object_deinit(&tmp);
+	return err;
+}
+
+struct drgn_error *linux_helper_idle_task(struct drgn_object *res, uint64_t cpu)
+{
+	struct drgn_error *err;
+	struct drgn_program *prog = drgn_object_program(res);
+
+	struct drgn_object tmp;
+	drgn_object_init(&tmp, prog);
+	err = drgn_program_find_object(prog, "runqueues", NULL,
+				       DRGN_FIND_OBJECT_ANY, &tmp);
+	if (err)
+		goto out;
+	err = drgn_object_address_of(&tmp, &tmp);
+	if (err)
+		goto out;
+	err = linux_helper_per_cpu_ptr(&tmp, &tmp, cpu);
+	if (err)
+		goto out;
+	err = drgn_object_member_dereference(res, &tmp, "idle");
+out:
+	drgn_object_deinit(&tmp);
+	return err;
+}
+
 struct drgn_error *
 linux_helper_radix_tree_lookup(struct drgn_object *res,
 			       const struct drgn_object *root, uint64_t index)
@@ -344,7 +407,6 @@ find_pid_in_pid_hash(struct drgn_object *res, const struct drgn_object *ns,
 			uint64_t addr, tmp_addr;
 			union drgn_value node_nr;
 			uint64_t node_ns;
-			char member[64];
 
 			err = drgn_object_read(&node, &node);
 			if (err)
@@ -381,8 +443,14 @@ find_pid_in_pid_hash(struct drgn_object *res, const struct drgn_object *ns,
 			if (node_ns != ns_addr)
 				goto next;
 
-			sprintf(member, "numbers[%" PRIu64 "].pid_chain",
-				ns_level.uvalue);
+#define FORMAT "numbers[%" PRIu64 "].pid_chain"
+			char member[sizeof(FORMAT)
+				    - sizeof("%" PRIu64)
+				    + max_decimal_length(uint64_t)
+				    + 1];
+			snprintf(member, sizeof(member), FORMAT,
+				 ns_level.uvalue);
+#undef FORMAT
 			err = drgn_object_container_of(res, &node,
 						       drgn_type_type(pidp_type.type),
 						       member);
@@ -451,7 +519,6 @@ struct drgn_error *linux_helper_pid_task(struct drgn_object *res,
 	struct drgn_qualified_type task_struct_type;
 	bool truthy;
 	struct drgn_object first;
-	char member[64];
 
 	drgn_object_init(&first, drgn_object_program(res));
 
@@ -486,12 +553,21 @@ struct drgn_error *linux_helper_pid_task(struct drgn_object *res,
 		goto null;
 
 	/* container_of(first, struct task_struct, pid_links[pid_type]) */
-	sprintf(member, "pid_links[%" PRIu64 "]", pid_type);
+#define PID_LINKS_FORMAT "pid_links[%" PRIu64 "]"
+#define PIDS_NODE_FORMAT "pids[%" PRIu64 "].node"
+	char member[max_iconst(sizeof(PID_LINKS_FORMAT),
+			       sizeof(PIDS_NODE_FORMAT))
+		    - sizeof("%" PRIu64)
+		    + max_decimal_length(uint64_t)
+		    + 1];
+	snprintf(member, sizeof(member), PID_LINKS_FORMAT, pid_type);
 	err = drgn_object_container_of(res, &first, task_struct_type, member);
 	if (err && err->code == DRGN_ERROR_LOOKUP) {
 		drgn_error_destroy(err);
 		/* container_of(first, struct task_struct, pids[pid_type].node) */
-		sprintf(member, "pids[%" PRIu64 "].node", pid_type);
+		snprintf(member, sizeof(member), PIDS_NODE_FORMAT, pid_type);
+#undef PID_LINKS_FORMAT
+#undef PIDS_NODE_FORMAT
 		err = drgn_object_container_of(res, &first, task_struct_type,
 					       member);
 	}
@@ -533,3 +609,80 @@ out:
 	drgn_object_deinit(&pid_obj);
 	return err;
 }
+
+struct drgn_error *
+linux_helper_task_iterator_init(struct linux_helper_task_iterator *it,
+				struct drgn_program *prog)
+{
+	struct drgn_error *err;
+	it->done = false;
+	drgn_object_init(&it->task, prog);
+	err = drgn_program_find_object(prog, "init_task", NULL,
+				       DRGN_FIND_OBJECT_VARIABLE, &it->task);
+	if (err)
+		goto err;
+	it->task_struct_type = drgn_object_qualified_type(&it->task);
+	err = drgn_object_address_of(&it->task, &it->task);
+	if (err)
+		goto err;
+	err = drgn_object_read_unsigned(&it->task, &it->init_task_address);
+	if (err)
+		goto err;
+	it->thread_group_address = it->init_task_address;
+	return NULL;
+
+err:
+	drgn_object_deinit(&it->task);
+	return err;
+}
+
+void linux_helper_task_iterator_deinit(struct linux_helper_task_iterator *it)
+{
+	drgn_object_deinit(&it->task);
+}
+
+struct drgn_error *
+linux_helper_task_iterator_next(struct linux_helper_task_iterator *it,
+				const struct drgn_object **ret)
+{
+	if (it->done) {
+		*ret = NULL;
+		return NULL;
+	}
+	struct drgn_error *err;
+	struct drgn_object *task = &it->task;
+	err = drgn_object_member_dereference(task, task, "thread_group");
+	if (err)
+		return err;
+	err = drgn_object_member(task, task, "next");
+	if (err)
+		return err;
+	err = drgn_object_container_of(task, task, it->task_struct_type,
+				       "thread_group");
+	if (err)
+		return err;
+	uint64_t task_address;
+	err = drgn_object_read_unsigned(task, &task_address);
+	if (err)
+		return err;
+	if (task_address == it->thread_group_address) {
+		err = drgn_object_member_dereference(task, task, "tasks");
+		if (err)
+			return err;
+		err = drgn_object_member(task, task, "next");
+		if (err)
+			return err;
+		err = drgn_object_container_of(task, task,
+					       it->task_struct_type, "tasks");
+		if (err)
+			return err;
+		err = drgn_object_read_unsigned(task,
+						&it->thread_group_address);
+		if (err)
+			return err;
+		if (it->thread_group_address == it->init_task_address)
+			it->done = true;
+	}
+	*ret = task;
+	return NULL;
+}
