@@ -3,20 +3,24 @@
 
 #include <assert.h>
 #include <byteswap.h>
+#include <dirent.h>
 #include <elf.h>
 #include <elfutils/libdw.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <gelf.h>
 #include <inttypes.h>
+#include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/statfs.h>
+#include <sys/types.h>
 #include <unistd.h>
 
 #include "debug_info.h"
 #include "error.h"
+#include "helpers.h"
 #include "language.h"
 #include "linux_kernel.h"
 #include "memory_reader.h"
@@ -27,8 +31,32 @@
 #include "vector.h"
 #include "util.h"
 
+static inline uint32_t drgn_thread_to_key(const struct drgn_thread *entry)
+{
+	return entry->tid;
+}
+
 DEFINE_VECTOR_FUNCTIONS(drgn_prstatus_vector)
-DEFINE_HASH_MAP_FUNCTIONS(drgn_prstatus_map, int_key_hash_pair, scalar_key_eq)
+DEFINE_HASH_TABLE_FUNCTIONS(drgn_thread_set, drgn_thread_to_key,
+			    int_key_hash_pair, scalar_key_eq)
+
+struct drgn_thread_iterator {
+	struct drgn_program *prog;
+	union {
+		/* For userspace core dumps. */
+		struct drgn_thread_set_iterator iterator;
+		struct {
+			union {
+				/* For live processes. */
+				DIR *tasks_dir;
+				/* For the Linux kernel. */
+				struct linux_helper_task_iterator task_iter;
+			};
+			/* For both live processes and the Linux kernel. */
+			struct drgn_thread entry;
+		};
+	};
+};
 
 LIBDRGN_PUBLIC enum drgn_program_flags
 drgn_program_flags(struct drgn_program *prog)
@@ -48,6 +76,12 @@ drgn_program_language(struct drgn_program *prog)
 	return prog->lang ? prog->lang : &drgn_default_language;
 }
 
+LIBDRGN_PUBLIC void drgn_program_set_language(struct drgn_program *prog,
+					      const struct drgn_language *lang)
+{
+	prog->lang = lang;
+}
+
 void drgn_program_set_platform(struct drgn_program *prog,
 			       const struct drgn_platform *platform)
 {
@@ -75,12 +109,21 @@ void drgn_program_init(struct drgn_program *prog,
 
 void drgn_program_deinit(struct drgn_program *prog)
 {
-	if (prog->prstatus_cached) {
+	if (prog->core_dump_notes_cached) {
 		if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
 			drgn_prstatus_vector_deinit(&prog->prstatus_vector);
 		else
-			drgn_prstatus_map_deinit(&prog->prstatus_map);
+			drgn_thread_set_deinit(&prog->thread_set);
 	}
+	/*
+	 * For userspace core dumps, main_thread and crashed_thread are in
+	 * prog->thread_set and thus freed by the above call to
+	 * drgn_thread_set_deinit().
+	 */
+	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
+		drgn_thread_destroy(prog->crashed_thread);
+	else if (prog->flags & DRGN_PROGRAM_IS_LIVE)
+		drgn_thread_destroy(prog->main_thread);
 	free(prog->pgtable_it);
 
 	drgn_object_deinit(&prog->vmemmap);
@@ -512,8 +555,10 @@ drgn_program_set_pid(struct drgn_program *prog, pid_t pid)
 	if (err)
 		return err;
 
-	char buf[64];
-	sprintf(buf, "/proc/%ld/mem", (long)pid);
+#define FORMAT "/proc/%ld/mem"
+	char buf[sizeof(FORMAT) - sizeof("%ld") + max_decimal_length(long) + 1];
+	snprintf(buf, sizeof(buf), FORMAT, (long)pid);
+#undef FORMAT
 	prog->core_fd = open(buf, O_RDONLY);
 	if (prog->core_fd == -1)
 		return drgn_error_create_os("open", errno, buf);
@@ -655,9 +700,88 @@ static struct drgn_error *get_prstatus_pid(struct drgn_program *prog, const char
 	return NULL;
 }
 
+static struct drgn_error *get_prpsinfo_pid(struct drgn_program *prog,
+					   const char *data, size_t size,
+					   uint32_t *ret)
+{
+	bool is_64_bit, bswap;
+	struct drgn_error *err = drgn_program_is_64_bit(prog, &is_64_bit);
+	if (err)
+		return err;
+	err = drgn_program_bswap(prog, &bswap);
+	if (err)
+		return err;
+
+	size_t offset = is_64_bit ? 24 : 12;
+	uint32_t pr_pid;
+	if (size < offset + sizeof(pr_pid)) {
+		return drgn_error_create(DRGN_ERROR_OTHER,
+					 "NT_PRPSINFO is truncated");
+	}
+	memcpy(&pr_pid, data + offset, sizeof(pr_pid));
+	if (bswap)
+		pr_pid = bswap_32(pr_pid);
+	*ret = pr_pid;
+	return NULL;
+}
+
+struct drgn_error *drgn_thread_dup_internal(const struct drgn_thread *thread,
+					    struct drgn_thread *ret)
+{
+	struct drgn_error *err = NULL;
+	ret->prog = thread->prog;
+	ret->tid = thread->tid;
+	/* Don't need a deep copy here since the PRSTATUS notes are cached. */
+	ret->prstatus = thread->prstatus;
+	if (thread->prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) {
+		drgn_object_init(&ret->object, thread->prog);
+		err = drgn_object_copy(&ret->object, &thread->object);
+		if (err)
+			drgn_object_deinit(&ret->object);
+	}
+	return err;
+}
+
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_thread_dup(const struct drgn_thread *thread, struct drgn_thread **ret)
+{
+	if (!(thread->prog->flags &
+	      (DRGN_PROGRAM_IS_LINUX_KERNEL | DRGN_PROGRAM_IS_LIVE))) {
+		/*
+		 * For userspace core dumps, all threads are cached and
+		 * immutable, so we can return the same handle.
+		 */
+		*ret = (struct drgn_thread *)thread;
+		return NULL;
+	}
+
+	*ret = malloc(sizeof(**ret));
+	if (!*ret)
+		return &drgn_enomem;
+	struct drgn_error *err = drgn_thread_dup_internal(thread, *ret);
+	if (err)
+		free(*ret);
+	return err;
+}
+
+void drgn_thread_deinit(struct drgn_thread *thread) {
+	if (thread->prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
+		drgn_object_deinit(&thread->object);
+}
+
+LIBDRGN_PUBLIC void drgn_thread_destroy(struct drgn_thread *thread)
+{
+	if (thread) {
+		drgn_thread_deinit(thread);
+		if (thread->prog->flags &
+		    (DRGN_PROGRAM_IS_LINUX_KERNEL | DRGN_PROGRAM_IS_LIVE))
+			free(thread);
+	}
+}
+
 struct drgn_error *drgn_program_cache_prstatus_entry(struct drgn_program *prog,
 						     const char *data,
-						     size_t size)
+						     size_t size, uint32_t *ret)
 {
 	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) {
 		struct nstring *entry =
@@ -667,36 +791,47 @@ struct drgn_error *drgn_program_cache_prstatus_entry(struct drgn_program *prog,
 		entry->str = data;
 		entry->len = size;
 	} else {
-		struct drgn_prstatus_map_entry entry = {
-			.value = { data, size },
-		};
+		struct drgn_thread thread = {
+			.prog = prog,
+			.prstatus = { data, size },
+	       };
 		struct drgn_error *err = get_prstatus_pid(prog, data, size,
-							  &entry.key);
+							  &thread.tid);
 		if (err)
 			return err;
-		if (drgn_prstatus_map_insert(&prog->prstatus_map, &entry,
-					     NULL) == -1)
+		*ret = thread.tid;
+		if (drgn_thread_set_insert(&prog->thread_set, &thread,
+					   NULL) == -1)
 			return &drgn_enomem;
 	}
 	return NULL;
 }
 
-static struct drgn_error *drgn_program_cache_prstatus(struct drgn_program *prog)
+static struct drgn_error *
+drgn_program_cache_core_dump_notes(struct drgn_program *prog)
 {
 	struct drgn_error *err;
 	size_t phnum, i;
+	bool found_prstatus = false;
+	uint32_t first_prstatus_tid;
+	bool found_prpsinfo = false;
+	uint32_t prpsinfo_pid;
 
-	if (prog->prstatus_cached)
+	if (prog->core_dump_notes_cached)
 		return NULL;
 
+	assert(!(prog->flags & DRGN_PROGRAM_IS_LIVE));
+
 	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
 		drgn_prstatus_vector_init(&prog->prstatus_vector);
 	else
-		drgn_prstatus_map_init(&prog->prstatus_map);
+		drgn_thread_set_init(&prog->thread_set);
 
 #ifdef WITH_LIBKDUMPFILE
 	if (prog->kdump_ctx) {
-		err = drgn_program_cache_prstatus_kdump(prog);
+		err = drgn_program_cache_kdump_notes(prog);
+		if (err)
+			goto err;
 		goto out;
 	}
 #endif
@@ -706,7 +841,7 @@ static struct drgn_error *drgn_program_cache_prstatus(struct drgn_program *prog)
 	}
 	if (elf_getphdrnum(prog->core, &phnum) != 0) {
 		err = drgn_error_libelf();
-		goto out;
+		goto err;
 	}
 	for (i = 0; i < phnum; i++) {
 		GElf_Phdr phdr_mem, *phdr;
@@ -718,7 +853,7 @@ static struct drgn_error *drgn_program_cache_prstatus(struct drgn_program *prog)
 		phdr = gelf_getphdr(prog->core, i, &phdr_mem);
 		if (!phdr) {
 			err = drgn_error_libelf();
-			goto out;
+			goto err;
 		}
 		if (phdr->p_type != PT_NOTE)
 			continue;
@@ -728,7 +863,7 @@ static struct drgn_error *drgn_program_cache_prstatus(struct drgn_program *prog)
 					    note_header_type(phdr->p_align));
 		if (!data) {
 			err = drgn_error_libelf();
-			goto out;
+			goto err;
 		}
 
 		offset = 0;
@@ -738,38 +873,454 @@ static struct drgn_error *drgn_program_cache_prstatus(struct drgn_program *prog)
 			const char *name;
 
 			name = (char *)data->d_buf + name_offset;
-			if (strncmp(name, "CORE", nhdr.n_namesz) != 0 ||
-			    nhdr.n_type != NT_PRSTATUS)
+			if (strncmp(name, "CORE", nhdr.n_namesz) != 0)
 				continue;
 
-			err = drgn_program_cache_prstatus_entry(prog,
-								(char *)data->d_buf + desc_offset,
-								nhdr.n_descsz);
-			if (err)
-				goto out;
+			if (nhdr.n_type == NT_PRPSINFO) {
+				err = get_prpsinfo_pid(prog,
+						       (char *)data->d_buf + desc_offset,
+						       nhdr.n_descsz,
+						       &prpsinfo_pid);
+				if (err)
+					goto err;
+				found_prpsinfo = true;
+			} else if (nhdr.n_type == NT_PRSTATUS) {
+				uint32_t tid;
+				err = drgn_program_cache_prstatus_entry(prog,
+						(char *)data->d_buf + desc_offset,
+						nhdr.n_descsz,
+						&tid);
+				if (err)
+					goto err;
+				/*
+				 * The first PRSTATUS note is the crashed thread. See
+				 * fs/binfmt_elf.c:fill_note_info in the Linux kernel
+				 * and bfd/elf.c:elfcore_grok_prstatus in BFD.
+				 */
+				if (!found_prstatus) {
+					found_prstatus = true;
+					first_prstatus_tid = tid;
+				}
+			}
 		}
 	}
 
-	err = NULL;
 out:
-	if (err) {
-		if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
-			drgn_prstatus_vector_deinit(&prog->prstatus_vector);
-		else
-			drgn_prstatus_map_deinit(&prog->prstatus_map);
-	} else {
-		prog->prstatus_cached = true;
+	prog->core_dump_notes_cached = true;
+	if (!(prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)) {
+		if (found_prpsinfo) {
+			struct drgn_thread_set_iterator it =
+				drgn_thread_set_search(&prog->thread_set,
+						       &prpsinfo_pid);
+			/* If the PID isn't found, then this is NULL. */
+			prog->main_thread = it.entry;
+		}
+		if (found_prstatus) {
+			/*
+			 * Now that thread_set won't be modified, look up the crashed
+			 * thread entry.
+			 */
+			struct drgn_thread_set_iterator it =
+				drgn_thread_set_search(&prog->thread_set,
+						       &first_prstatus_tid);
+			assert(it.entry);
+			prog->crashed_thread = it.entry;
+		}
 	}
+	return NULL;
+
+err:
+	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
+		drgn_prstatus_vector_deinit(&prog->prstatus_vector);
+	else
+		drgn_thread_set_deinit(&prog->thread_set);
 	return err;
 }
 
+static struct drgn_error *
+drgn_thread_iterator_init_linux_kernel(struct drgn_thread_iterator *it)
+{
+	struct drgn_error *err = linux_helper_task_iterator_init(&it->task_iter,
+								 it->prog);
+	if (err)
+		return err;
+	drgn_object_init(&it->entry.object, it->prog);
+	it->entry.prstatus = (struct nstring){};
+	return NULL;
+}
+
+static struct drgn_error *
+drgn_thread_iterator_init_userspace_live(struct drgn_thread_iterator *it)
+{
+#define FORMAT "/proc/%ld/task"
+	char path[sizeof(FORMAT)
+		- sizeof("%ld")
+		+ max_decimal_length(long)
+		+ 1];
+	snprintf(path, sizeof(path), FORMAT, (long)it->prog->pid);
+#undef FORMAT
+	it->tasks_dir = opendir(path);
+	if (!it->tasks_dir)
+		return drgn_error_create_os("opendir", errno, path);
+	it->entry.prog = it->prog;
+	it->entry.prstatus = (struct nstring){};
+	return NULL;
+}
+
+static struct drgn_error *
+drgn_thread_iterator_init_userspace_core(struct drgn_thread_iterator *it)
+{
+	struct drgn_error *err = drgn_program_cache_core_dump_notes(it->prog);
+	if (err)
+		return err;
+	it->iterator = drgn_thread_set_first(&it->prog->thread_set);
+	return NULL;
+}
+
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_thread_iterator_create(struct drgn_program *prog,
+			    struct drgn_thread_iterator **ret)
+{
+	struct drgn_error *err;
+
+	*ret = malloc(sizeof(**ret));
+	if (!*ret)
+		return &drgn_enomem;
+	(*ret)->prog = prog;
+	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
+		err = drgn_thread_iterator_init_linux_kernel(*ret);
+	else if (prog->flags & DRGN_PROGRAM_IS_LIVE)
+		err = drgn_thread_iterator_init_userspace_live(*ret);
+	else
+		err = drgn_thread_iterator_init_userspace_core(*ret);
+	if (err)
+		free(*ret);
+	return err;
+}
+
+LIBDRGN_PUBLIC void
+drgn_thread_iterator_destroy(struct drgn_thread_iterator *it)
+{
+	if (it) {
+		if (it->prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) {
+			drgn_object_deinit(&it->entry.object);
+			linux_helper_task_iterator_deinit(&it->task_iter);
+		} else if (it->prog->flags & DRGN_PROGRAM_IS_LIVE) {
+			closedir(it->tasks_dir);
+		}
+		free(it);
+	}
+}
+
+static struct drgn_error *
+drgn_thread_iterator_next_linux_kernel(struct drgn_thread_iterator *it,
+				       struct drgn_thread **ret)
+{
+	struct drgn_error *err;
+	const struct drgn_object *task;
+	err = linux_helper_task_iterator_next(&it->task_iter, &task);
+	if (err)
+		return err;
+	if (!task) {
+		*ret = NULL;
+		return NULL;
+	}
+	it->entry.prog = drgn_object_program(task);
+	err = drgn_object_copy(&it->entry.object, task);
+	if (err)
+		return err;
+	struct drgn_object tid;
+	union drgn_value tid_value;
+	drgn_object_init(&tid, drgn_object_program(task));
+	err = drgn_object_member_dereference(&tid, task, "pid");
+	if (!err)
+		err = drgn_object_read_integer(&tid, &tid_value);
+	drgn_object_deinit(&tid);
+	if (err)
+		return err;
+	it->entry.tid = tid_value.uvalue;
+	*ret = &it->entry;
+	return NULL;
+}
+
+static struct drgn_error *
+drgn_thread_iterator_next_userspace_live(struct drgn_thread_iterator *it,
+					 struct drgn_thread **ret)
+{
+	struct dirent *task;
+	unsigned long tid;
+	char *end;
+	do {
+		errno = 0;
+		task = readdir(it->tasks_dir);
+		if (!task) {
+			if (errno) {
+				return drgn_error_create_os("readdir", errno,
+							    NULL);
+			}
+			*ret = NULL;
+			return NULL;
+		}
+
+		errno = 0;
+		tid = strtoul(task->d_name, &end, 10);
+		/*
+		 * Skip anything that isn't a number (like "." and "..") or
+		 * overflows (which is impossible normally).
+		 */
+	} while (*end != '\0' || (tid == ULONG_MAX && errno == ERANGE));
+	it->entry.tid = tid;
+	*ret = &it->entry;
+	return NULL;
+}
+
+static void
+drgn_thread_iterator_next_userspace_core(struct drgn_thread_iterator *it,
+					 struct drgn_thread **ret)
+{
+	*ret = it->iterator.entry;
+	if (it->iterator.entry)
+		it->iterator = drgn_thread_set_next(it->iterator);
+}
+
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_thread_iterator_next(struct drgn_thread_iterator *it,
+			  struct drgn_thread **ret)
+{
+	if (it->prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) {
+		return drgn_thread_iterator_next_linux_kernel(it, ret);
+	} else if (it->prog->flags & DRGN_PROGRAM_IS_LIVE) {
+		return drgn_thread_iterator_next_userspace_live(it, ret);
+	} else {
+		drgn_thread_iterator_next_userspace_core(it, ret);
+		return NULL;
+	}
+}
+
+static struct drgn_error *
+drgn_program_find_thread_linux_kernel(struct drgn_program *prog, uint32_t tid,
+				      struct drgn_thread **ret)
+{
+	struct drgn_error *err;
+
+	*ret = malloc(sizeof(**ret));
+	if (!*ret)
+		return &drgn_enomem;
+	(*ret)->prog = prog;
+	(*ret)->tid = tid;
+	(*ret)->prstatus = (struct nstring){};
+
+	struct drgn_object *object = &(*ret)->object;
+	drgn_object_init(object, prog);
+	err = drgn_program_find_object(prog, "init_pid_ns", NULL,
+				       DRGN_FIND_OBJECT_VARIABLE, object);
+	if (err)
+		goto err;
+	err = drgn_object_address_of(object, object);
+	if (err)
+		goto err;
+	err = linux_helper_find_task(object, object, tid);
+	if (err)
+		goto err;
+	bool truthy;
+	err = drgn_object_bool(object, &truthy);
+	if (err)
+		goto err;
+	if (!truthy) {
+		drgn_thread_destroy(*ret);
+		*ret = NULL;
+	}
+	return NULL;
+
+err:
+	drgn_thread_destroy(*ret);
+	return err;
+}
+
+static struct drgn_error *
+drgn_program_find_thread_userspace_live(struct drgn_program *prog, uint32_t tid,
+					struct drgn_thread **ret)
+{
+#define FORMAT "/proc/%ld/task/%" PRIu32
+	char path[sizeof(FORMAT)
+		- sizeof("%ld%" PRIu32)
+		+ max_decimal_length(long)
+		+ max_decimal_length(uint32_t)
+		+ 1];
+	snprintf(path, sizeof(path), FORMAT, (long)prog->pid, tid);
+#undef FORMAT
+	int r = access(path, F_OK);
+	if (r == 0) {
+		*ret = malloc(sizeof(**ret));
+		if (!*ret)
+			return &drgn_enomem;
+		(*ret)->prog = prog;
+		(*ret)->tid = tid;
+		(*ret)->prstatus = (struct nstring){};
+		return NULL;
+	} else if (errno == ENOENT) {
+		*ret = NULL;
+		return NULL;
+	} else {
+		return drgn_error_create_os("access", errno, path);
+	}
+}
+
+static struct drgn_error *
+drgn_program_find_thread_userspace_core(struct drgn_program *prog, uint32_t tid,
+					struct drgn_thread **ret)
+{
+	struct drgn_error *err = drgn_program_cache_core_dump_notes(prog);
+	if (err)
+		return err;
+	*ret = drgn_thread_set_search(&prog->thread_set, &tid).entry;
+	return NULL;
+}
+
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_program_find_thread(struct drgn_program *prog, uint32_t tid,
+			 struct drgn_thread **ret)
+{
+	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
+		return drgn_program_find_thread_linux_kernel(prog, tid, ret);
+	else if (prog->flags & DRGN_PROGRAM_IS_LIVE)
+		return drgn_program_find_thread_userspace_live(prog, tid, ret);
+	else
+		return drgn_program_find_thread_userspace_core(prog, tid, ret);
+}
+
+static struct drgn_error *
+drgn_program_kernel_core_dump_cache_crashed_thread(struct drgn_program *prog)
+{
+	struct drgn_error *err;
+
+	assert((prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) &&
+	       !(prog->flags & DRGN_PROGRAM_IS_LIVE));
+	assert(prog->core_dump_notes_cached);
+	if (prog->crashed_thread)
+		return NULL;
+
+	struct drgn_object crashing_cpu;
+	union drgn_value crashing_cpu_value;
+	drgn_object_init(&crashing_cpu, prog);
+	err = drgn_program_find_object(prog, "crashing_cpu", NULL,
+				       DRGN_FIND_OBJECT_VARIABLE,
+				       &crashing_cpu);
+	if (!err) {
+		err = drgn_object_read_integer(&crashing_cpu,
+					       &crashing_cpu_value);
+	}
+	drgn_object_deinit(&crashing_cpu);
+	if (err || crashing_cpu_value.uvalue >= prog->prstatus_vector.size)
+		return err;
+
+	struct nstring *prstatus =
+		&prog->prstatus_vector.data[crashing_cpu_value.uvalue];
+	uint32_t crashed_thread_tid;
+	err = get_prstatus_pid(prog, prstatus->str, prstatus->len,
+			       &crashed_thread_tid);
+	if (err)
+		return err;
+
+	if (crashed_thread_tid == 0) {
+		prog->crashed_thread = malloc(sizeof(*prog->crashed_thread));
+		if (!prog->crashed_thread)
+			return &drgn_enomem;
+		prog->crashed_thread->prog = prog;
+		prog->crashed_thread->tid = crashed_thread_tid;
+		drgn_object_init(&prog->crashed_thread->object, prog);
+		err = linux_helper_idle_task(&prog->crashed_thread->object,
+					     crashing_cpu_value.uvalue);
+		if (err) {
+			drgn_object_deinit(&prog->crashed_thread->object);
+			free(prog->crashed_thread);
+		}
+	} else {
+		err = drgn_program_find_thread(prog, crashed_thread_tid,
+					       &prog->crashed_thread);
+	}
+	if (err) {
+		prog->crashed_thread = NULL;
+		return err;
+	}
+	prog->crashed_thread->prstatus = *prstatus;
+	return NULL;
+}
+
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_program_main_thread(struct drgn_program *prog, struct drgn_thread **ret)
+{
+	struct drgn_error *err;
+
+	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) {
+		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+					 "main thread is not defined for the Linux kernel");
+	}
+	if (prog->flags & DRGN_PROGRAM_IS_LIVE) {
+		if (!prog->main_thread) {
+			err = drgn_program_find_thread(prog, prog->pid,
+						       &prog->main_thread);
+			if (err) {
+				prog->main_thread = NULL;
+				return err;
+			}
+		}
+	} else {
+		err = drgn_program_cache_core_dump_notes(prog);
+		if (err)
+			return err;
+	}
+	if (!prog->main_thread) {
+		return drgn_error_create(DRGN_ERROR_OTHER,
+					 "main thread not found");
+	}
+	*ret = prog->main_thread;
+	return NULL;
+}
+
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_program_crashed_thread(struct drgn_program *prog, struct drgn_thread **ret)
+{
+	struct drgn_error *err;
+
+	if (prog->flags & DRGN_PROGRAM_IS_LIVE) {
+		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+					 "crashed thread is only defined for core dumps");
+	}
+	err = drgn_program_cache_core_dump_notes(prog);
+	if (err)
+		return err;
+	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) {
+		err = drgn_program_kernel_core_dump_cache_crashed_thread(prog);
+		if (err)
+			return err;
+	}
+	if (!prog->crashed_thread) {
+		return drgn_error_create(DRGN_ERROR_OTHER,
+					 "crashed thread not found");
+	}
+	*ret = prog->crashed_thread;
+	return NULL;
+}
+
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_thread_object(struct drgn_thread *thread, const struct drgn_object **ret)
+{
+	if (!(thread->prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)) {
+		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+					 "thread object is currently only defined for the Linux kernel");
+	}
+	*ret = &thread->object;
+	return NULL;
+}
+
 struct drgn_error *drgn_program_find_prstatus_by_cpu(struct drgn_program *prog,
 						     uint32_t cpu,
 						     struct nstring *ret,
 						     uint32_t *tid_ret)
 {
 	assert(prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL);
-	struct drgn_error *err = drgn_program_cache_prstatus(prog);
+	struct drgn_error *err = drgn_program_cache_core_dump_notes(prog);
 	if (err)
 		return err;
 
@@ -788,20 +1339,19 @@ struct drgn_error *drgn_program_find_prstatus_by_tid(struct drgn_program *prog,
 						     struct nstring *ret)
 {
 	struct drgn_error *err;
-	struct drgn_prstatus_map_iterator it;
 
 	assert(!(prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL));
-	err = drgn_program_cache_prstatus(prog);
+	struct drgn_thread *thread;
+	err = drgn_program_find_thread(prog, tid, &thread);
 	if (err)
 		return err;
 
-	it = drgn_prstatus_map_search(&prog->prstatus_map, &tid);
-	if (!it.entry) {
+	if (!thread) {
 		ret->str = NULL;
 		ret->len = 0;
 		return NULL;
 	}
-	*ret = it.entry->value;
+	*ret = thread->prstatus;
 	return NULL;
 }
 
@@ -1108,6 +1658,132 @@ drgn_program_find_symbol_by_address(struct drgn_program *prog, uint64_t address,
 	return NULL;
 }
 
+DEFINE_VECTOR(symbolp_vector, struct drgn_symbol *)
+
+enum {
+	SYMBOLS_SEARCH_NAME = (1 << 0),
+	SYMBOLS_SEARCH_ADDRESS = (1 << 1),
+	SYMBOLS_SEARCH_ALL = (1 << 2),
+};
+
+struct symbols_search_arg {
+	const char *name;
+	uint64_t address;
+	struct symbolp_vector results;
+	unsigned int flags;
+};
+
+static bool symbol_match(struct symbols_search_arg *arg, GElf_Addr addr,
+			 const GElf_Sym *sym, const char *name)
+{
+	if (arg->flags & SYMBOLS_SEARCH_ALL)
+		return true;
+	if ((arg->flags & SYMBOLS_SEARCH_NAME) && strcmp(name, arg->name) == 0)
+		return true;
+	if ((arg->flags & SYMBOLS_SEARCH_ADDRESS) &&
+	    arg->address >= addr && arg->address < addr + sym->st_size)
+		return true;
+	return false;
+}
+
+static int symbols_search_cb(Dwfl_Module *dwfl_module, void **userdatap,
+			     const char *module_name, Dwarf_Addr base,
+			     void *cb_arg)
+{
+	struct symbols_search_arg *arg = cb_arg;
+
+	int symtab_len = dwfl_module_getsymtab(dwfl_module);
+	if (symtab_len == -1)
+		return DWARF_CB_OK;
+
+	/* Ignore the zeroth null symbol */
+	for (int i = 1; i < symtab_len; i++) {
+		GElf_Sym elf_sym;
+		GElf_Addr elf_addr;
+		const char *name = dwfl_module_getsym_info(dwfl_module, i,
+							   &elf_sym, &elf_addr,
+							   NULL, NULL, NULL);
+		if (!name || !symbol_match(arg, elf_addr, &elf_sym, name))
+			continue;
+
+		struct drgn_symbol *sym = malloc(sizeof(*sym));
+		if (!sym)
+			return DWARF_CB_ABORT;
+		drgn_symbol_from_elf(name, elf_addr, &elf_sym, sym);
+		if (!symbolp_vector_append(&arg->results, &sym)) {
+			drgn_symbol_destroy(sym);
+			return DWARF_CB_ABORT;
+		}
+	}
+	return DWARF_CB_OK;
+}
+
+static struct drgn_error *
+symbols_search(struct drgn_program *prog, struct symbols_search_arg *arg,
+	       struct drgn_symbol ***syms_ret, size_t *count_ret)
+{
+	struct drgn_error *err;
+
+	if (!prog->dbinfo) {
+		return drgn_error_create(DRGN_ERROR_MISSING_DEBUG_INFO,
+					 "could not find matching symbols");
+	}
+
+	symbolp_vector_init(&arg->results);
+
+	/*
+	 * When searching for addresses, we can identify the exact module to
+	 * search. Otherwise we need to fall back to an exhaustive search.
+	 */
+	err = NULL;
+	if (arg->flags & SYMBOLS_SEARCH_ADDRESS) {
+		Dwfl_Module *module = dwfl_addrmodule(prog->dbinfo->dwfl,
+						      arg->address);
+		if (module && symbols_search_cb(module, NULL, NULL, 0, arg))
+			err = &drgn_enomem;
+	} else {
+		if (dwfl_getmodules(prog->dbinfo->dwfl, symbols_search_cb, arg,
+				    0))
+			err = &drgn_enomem;
+	}
+
+	if (err) {
+		for (size_t i = 0; i < arg->results.size; i++)
+			drgn_symbol_destroy(arg->results.data[i]);
+		symbolp_vector_deinit(&arg->results);
+	} else {
+		symbolp_vector_shrink_to_fit(&arg->results);
+		*count_ret = arg->results.size;
+		*syms_ret = arg->results.data;
+	}
+	return err;
+}
+
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_program_find_symbols_by_name(struct drgn_program *prog, const char *name,
+				  struct drgn_symbol ***syms_ret,
+				  size_t *count_ret)
+{
+	struct symbols_search_arg arg = {
+		.name = name,
+		.flags = name ? SYMBOLS_SEARCH_NAME : SYMBOLS_SEARCH_ALL,
+	};
+	return symbols_search(prog, &arg, syms_ret, count_ret);
+}
+
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_program_find_symbols_by_address(struct drgn_program *prog,
+				     uint64_t address,
+				     struct drgn_symbol ***syms_ret,
+				     size_t *count_ret)
+{
+	struct symbols_search_arg arg = {
+		.address = address,
+		.flags = SYMBOLS_SEARCH_ADDRESS,
+	};
+	return symbols_search(prog, &arg, syms_ret, count_ret);
+}
+
 struct find_symbol_by_name_arg {
 	const char *name;
 	GElf_Sym sym;
