@@ -97,6 +97,17 @@ typedef struct {
 	struct pyobjectp_set objects;
 } Program;
 
+typedef struct {
+	PyObject_HEAD
+	struct drgn_thread thread;
+} Thread;
+
+typedef struct {
+	PyObject_HEAD
+	Program *prog;
+	struct drgn_thread_iterator *iterator;
+} ThreadIterator;
+
 typedef struct {
 	PyObject_HEAD
 	const struct drgn_register *reg;
@@ -175,6 +186,8 @@ extern PyTypeObject Register_type;
 extern PyTypeObject StackFrame_type;
 extern PyTypeObject StackTrace_type;
 extern PyTypeObject Symbol_type;
+extern PyTypeObject Thread_type;
+extern PyTypeObject ThreadIterator_type;
 extern PyTypeObject TypeEnumerator_type;
 extern PyTypeObject TypeMember_type;
 extern PyTypeObject TypeParameter_type;
@@ -229,6 +242,10 @@ Program *program_from_pid(PyObject *self, PyObject *args, PyObject *kwds);
 
 PyObject *Symbol_wrap(struct drgn_symbol *sym, Program *prog);
 
+PyObject *Thread_wrap(struct drgn_thread *drgn_thread);
+
+PyObject *StackTrace_wrap(struct drgn_stack_trace *trace);
+
 static inline Program *DrgnType_prog(DrgnType *type)
 {
 	return container_of(drgn_type_program(type->type), Program, prog);
@@ -282,6 +299,10 @@ int enum_converter(PyObject *o, void *p);
 
 PyObject *drgnpy_linux_helper_read_vm(PyObject *self, PyObject *args,
 				      PyObject *kwds);
+DrgnObject *drgnpy_linux_helper_per_cpu_ptr(PyObject *self, PyObject *args,
+					    PyObject *kwds);
+DrgnObject *drgnpy_linux_helper_idle_task(PyObject *self, PyObject *args,
+					  PyObject *kwds);
 DrgnObject *drgnpy_linux_helper_radix_tree_lookup(PyObject *self,
 						  PyObject *args,
 						  PyObject *kwds);
