@@ -124,6 +124,11 @@ static PyMethodDef drgn_methods[] = {
 	 METH_VARARGS | METH_KEYWORDS, drgn_program_from_pid_DOC},
 	{"_linux_helper_read_vm", (PyCFunction)drgnpy_linux_helper_read_vm,
 	 METH_VARARGS | METH_KEYWORDS},
+	{"_linux_helper_per_cpu_ptr",
+	 (PyCFunction)drgnpy_linux_helper_per_cpu_ptr,
+	 METH_VARARGS | METH_KEYWORDS},
+	{"_linux_helper_idle_task", (PyCFunction)drgnpy_linux_helper_idle_task,
+	 METH_VARARGS | METH_KEYWORDS},
 	{"_linux_helper_radix_tree_lookup",
 	 (PyCFunction)drgnpy_linux_helper_radix_tree_lookup,
 	 METH_VARARGS | METH_KEYWORDS},
@@ -230,6 +235,8 @@ DRGNPY_PUBLIC PyMODINIT_FUNC PyInit__drgn(void)
 	    add_type(m, &StackTrace_type) ||
 	    add_type(m, &Symbol_type) ||
 	    add_type(m, &DrgnType_type) ||
+	    add_type(m, &Thread_type) ||
+	    add_type(m, &ThreadIterator_type) ||
 	    add_type(m, &TypeEnumerator_type) ||
 	    add_type(m, &TypeMember_type) ||
 	    add_type(m, &TypeParameter_type) ||
